#include <stdio.h>
#include <stdlib.h>
#include <math.h>





typedef struct{                // BMP file header structure
  unsigned short bfType;       // Magic number for file
  unsigned int   bfSize;       // Size of file
  unsigned short bfReserved1;  // Reserved
  unsigned short bfReserved2;  // ... 
  unsigned int   bfOffBits;    // Offset to bitmap data
} BITMAPFILEHEADER;


#  define BF_TYPE 0x4D42       // "MB"


typedef struct{                // BMP file info structure
  unsigned int   biSize;       // Size of info header
  int      biWidth;            // Width of image
  int      biHeight;           // Height of image
  unsigned short biPlanes;     // Number of color planes
  unsigned short biBitCount;   // Number of bits per pixel
  unsigned int   biCompression;// Type of compression to use
  unsigned int   biSizeImage;  // Size of image data
  int      biXPelsPerMeter;    // X pixels per meter
  int      biYPelsPerMeter;    // Y pixels per meter
  unsigned int  biClrUsed;     // Number of colors used
  unsigned int  biClrImportant;// Number of important colors
} BITMAPINFOHEADER;


typedef struct{                // Colormap entry structure
  unsigned char  rgbBlue;      // Blue value
  unsigned char  rgbGreen;     // Green value
  unsigned char  rgbRed;       // Red value
  unsigned char  rgbReserved;  // Reserved
} RGBQUAD;


typedef struct{                // Bitmap information structure
  BITMAPINFOHEADER bmiHeader;  // Image header
  RGBQUAD      bmiColors[256]; // Image colormap
} BITMAPINFO;


unsigned short read_word(FILE *fp) {
  unsigned char b0, b1;
  b0 = getc(fp);
  b1 = getc(fp);
  return ((b1 << 8) | b0);
}


unsigned int read_dword(FILE *fp) {
  unsigned char b0, b1, b2, b3;
  b0 = getc(fp);
  b1 = getc(fp);
  b2 = getc(fp);
  b3 = getc(fp);
  return ((((((b3 << 8) | b2) << 8) | b1) << 8) | b0);
}


int read_long(FILE *fp) {
  unsigned char b0, b1, b2, b3;
  b0 = getc(fp);
  b1 = getc(fp);
  b2 = getc(fp);
  b3 = getc(fp);
  return ((int)(((((b3 << 8) | b2) << 8) | b1) << 8) | b0);
}



void printUsage(){
  printf("\nUsage: bmp2bin <infile> <outfile>\n\n");
  printf("       infile      ....  name of infile\n");
  printf("       outfile     ....  name of outfile (use \"-\" for stdout)\n");
}



int main(int argc, char *argv[]) {
  FILE            *fp;          // Open file pointer
  char            *bits,        // Bitmap pixel bits
                  *ptr,         // Pointer into bitmap
                  *final;       
  int             x, y,         // X and Y position in image
                  length,       // Line length
                  bitsize,      // Size of bitmap
                  infosize,     // Size of header information
                  i,j,b,
                  brightness,
                  point,
                  LGRAY=1,
                  DGRAY=0;
  unsigned char   *charmap,
                  buffer[10];
  BITMAPFILEHEADER header;
  BITMAPINFO *info;

  if(argv[2][0]!='-') {
    printf("\nBmp2Bin v1.00\n");
    printf("Copyright 2002 Michael Holt (michaelholt@knology.net)\n");
    printf("In: %s, Out: %s",argv[1],argv[2]);
    }
  
  if(argc!=3) {
    printUsage();
    return 1;
  }
  
  if ((fp = fopen(argv[1], "rb")) == NULL){
    printf("\nThis file does not exist.\n");
    printUsage();
    return 1;
  }

  header.bfType      = read_word(fp);
  header.bfSize      = read_dword(fp);
  header.bfReserved1 = read_word(fp);
  header.bfReserved2 = read_word(fp);
  header.bfOffBits   = read_dword(fp);

  if (header.bfType != BF_TYPE){
    printf("\nThis file is not a Bitmap.\n");
    fclose(fp);
    printUsage();
    return 1;
  }

  infosize = header.bfOffBits - 18;
  info = (BITMAPINFO*)malloc(sizeof(BITMAPINFO));
  if (info == NULL){
    printf("\nNot enough free memory.\n");
    fclose(fp);
    return 1;
  }

  info->bmiHeader.biSize          = read_dword(fp);
  info->bmiHeader.biWidth         = read_long(fp);
  info->bmiHeader.biHeight        = read_long(fp);
  info->bmiHeader.biPlanes        = read_word(fp);
  info->bmiHeader.biBitCount      = read_word(fp);
  info->bmiHeader.biCompression   = read_dword(fp);
  info->bmiHeader.biSizeImage     = read_dword(fp);
  info->bmiHeader.biXPelsPerMeter = read_long(fp);
  info->bmiHeader.biYPelsPerMeter = read_long(fp);
  info->bmiHeader.biClrUsed       = read_dword(fp);
  info->bmiHeader.biClrImportant  = read_dword(fp);

  if (infosize > 40)
    if (fread(info->bmiColors, infosize - 40, 1, fp) < 1){
      printf("\nCouldn't read header.\n");
      free(info);
      fclose(fp);
      printUsage();
      return 1;
    }

  if(info->bmiHeader.biBitCount!=24){
    printf("\nOnly 24 bit Bitmaps are supported.\n");
    free(info);
    fclose(fp);
    return 1;
  }

  if(info->bmiHeader.biCompression){
    printf("\nCompressed Bitmaps are not supported.\n");
    free(info);
    fclose(fp);
    return 1;
  }

  if ((bitsize = info->bmiHeader.biSizeImage) == 0)
    bitsize = (info->bmiHeader.biWidth *
           info->bmiHeader.biBitCount + 7) / 8 *
  	       abs(info->bmiHeader.biHeight);

  bits = (char*)malloc(bitsize);
  if (bits == NULL){
    printf("\nCouldn't allocate memory.\n");
    free(info);
    fclose(fp);
    return 1;
  }

  if (fread(bits, 1, bitsize, fp) < bitsize){
    printf("\nCouldn't read bitmap.\n");
    free(info);
    free(bits);
    fclose(fp);
    printUsage();
    return 1;
  }

  fclose(fp);

  length = (info->bmiHeader.biWidth * 3 + 3) & ~3;

  charmap = (unsigned char*)malloc(info->bmiHeader.biWidth*info->bmiHeader.biHeight);
  if(charmap==NULL) return 1;
  point=0;
  for (y = info->bmiHeader.biHeight-1; y >=0 ; y --){
    for (ptr = bits + y * length, x = info->bmiHeader.biWidth; x > 0; x --, ptr += 3){
      brightness = ((unsigned char)ptr[0]+(unsigned char)ptr[1]+(unsigned char)ptr[2])/3;
      charmap[point] = brightness; point++;
    }
  }


  final = (char*)malloc(info->bmiHeader.biWidth*info->bmiHeader.biHeight/4+1);
  point=0;
  for(LGRAY=1,DGRAY=0;DGRAY<2;LGRAY--,DGRAY++){
    for(i=0;i<info->bmiHeader.biWidth*info->bmiHeader.biHeight;i+=1){
      if(charmap[i]<64) buffer[i%8]=1;
      if(charmap[i]>=64&& charmap[i]<=128) buffer[i%8]=DGRAY;
      if(charmap[i]>128 && charmap[i]<196) buffer[i%8]=LGRAY;
      if(charmap[i]>196) buffer[i%8]=0;
      if(i%8==7){
        b=0;
        for(j=0;j<8;j++){
          b+=pow(2,7-j)*(buffer[j]);}
        final[point]=(char)b;
        point++;
      }
    }
  }

  if(argv[2][0]!='-') fp = fopen(argv[2],"wb");
  for(i=0;i<info->bmiHeader.biWidth*info->bmiHeader.biHeight/4;i++){
    if(argv[2][0]!='-') fwrite(final+i,1,1,fp);
    else printf("%c",final[i]);
  }
  if(argv[2][0]!='-') fclose(fp);
  
  if(argv[2][0]!='-') printf("\nBitmap [%s] converted Successfully.\n",argv[1]);

  free(bits);
  free(info);
  free(buffer);
  return 0;
}

