#include <stdio.h>
#include <limits.h>

#define MAX_UNIQUE_BUTTONS 3
#define MAX_BUTTONS 11
#define MAX_DOORS (MAX_BUTTONS*2)
#define MAX_SPRITES 25
#define MAX_TEXTURES 16
#define NR_TEXTURES 127

#if !(SHRT_MAX == 32767)
#error No 2 byte storage types found.
#endif

//#define DEBUG

#include "levstruct.h"
#include "walls_animated.h"
#include "levcodearray.h"

#define WALLBUTTONFLAG 64 //0b01000000

#define FlipInt(a) ((short)(a<<8)|(a>>8))
#define GetInt(a) {a = atoi(levplace);}
#define GetCharToPtr(a) {*(a) = (char)atoi(levplace);}
#define GetChar(a) {a = (char)atoi(levplace);}
#define GetUChar(a) {a = (unsigned char)atoi(levplace);}
#define GetIntTI(a) {a = FlipInt(atoi(levplace));}
#define LevKeepGoing() {levplace = SkipStuff(levplace);}
#define InfoKeepGoing() {infoplace = SkipInfoStuff(infoplace);}
#define FixString(s)                                    \
c=0;                                                    \
while(s[c])                                             \
{                                                       \
 if(s[c]=='\n' || s[c]=='\r' || s[c]=='\t')             \
 {                                                      \
  s[c]=0;                                               \
  break;                                                \
 }                                                      \
 c++;                                                   \
}

#ifdef COMPRESSED_TEXTURES
#define QuitOnError() \
if(out) fclose(out);\
if(txt) fclose(txt);\
if(in)  fclose(in );\
if(header) free(header);  \
for(a=0;a<total_maps;a++)       \
  if(dataptrs[a]) free(dataptrs[a]);  \
if(infobuffer) free(infobuffer);      \
if(levbuffer) free(levbuffer);       \
for(a=0;a<total_maps;a++)              \
  if(texarray[a]) free(texarray[a]);   \
return 1;
#else
#define QuitOnError() \
if(out) fclose(out);\
if(txt) fclose(txt);\
if(in)  fclose(in );\
if(header) free(header);  \
for(a=0;a<total_maps;a++)       \
  if(dataptrs[a]) free(dataptrs[a]);  \
if(infobuffer) free(infobuffer);      \
if(levbuffer) free(levbuffer);       \
return 1;
#endif

typedef struct
{
	unsigned char x;
	unsigned char y;
	unsigned char buttoncode;
} DoorInfo;
typedef struct
{
	unsigned char x;
	unsigned char y;
	unsigned char doorx;
	unsigned char doory;
	unsigned char buttoncode;
} ButtonInfo;

typedef struct
{
	unsigned char x;
	unsigned char y;
} DInfo;
typedef struct
{
	unsigned char x;
	unsigned char y;
	unsigned char doorx;
	unsigned char doory;
} BInfo;


char* SkipStuff(char* token) {
   while (!(*token==',')) token++;
   while ((*token==',')) token++;
   return token;
}
char* SkipInfoStuff(char* token) {
   while (!(*token=='\t')) token++;
   while ((*token=='\t')) token++;
   return token;
}
char* SkipLine(char *token)
{
   while (!(*token=='\n')) token++;
   token++;
   return token;
}


short ParseCeiling(char *str)
{
 short a = 0;
 while(a<BG_COUNT)
 {

  if(!strcmp(str,Backgrounds[a])) break;
  a++;
 }
 if(a != BG_COUNT) return a;
 else return WHITE; //Background WHITE is default
}

short ParseFloor(char *str)
{
 short a = 0;
 while(a<BG_COUNT)
 {

  if(!strcmp(str,Backgrounds[a])) break;
  a++;
 }
 if(a != BG_COUNT) return a;
 else return DARK; //Background DARKGRAY is default
}


int main(int argc, char *argv[])
{
unsigned long size = 0;
short total_maps = 0;
short sprite_count = 0;
short a,b,c;
short curx,cury;
short curdoor;
short curbutton;
char *levbuffer;
char *infobuffer;
char instr[] = "maps.txt";
char textstr[100];
char outstr[50];
char onelev[255];
char used_textures[NR_TEXTURES];
char used_buttons[NR_TEXTURES];
short used_tex,used_button;
short tmpcount;
char tile;
unsigned char mapcode;
char *levplace;
char *infoplace;
FILE *in=NULL,*out=NULL,*txt=NULL;
MAP_HEADER *header;
char wait[5];

printf("\nLevel Creator\nBy Malcolm Smith\n");

if(argv[0]) strcpy(textstr,argv[0]);
txt = fopen(textstr,"rt");
if(!txt)
{
  strcpy(textstr,"data.txt");
  txt = fopen(textstr,"rt");
  if(!txt)
  {
    printf("\nData File Name?\n");
    gets(textstr);
    txt = fopen(textstr,"rt");
    if(!txt) return 1;
  }
}
#ifdef DEBUG
printf("\nTXT File Opened");
#endif
while(!feof(txt))
{
 fgetc(txt);
 size++;
}
rewind(txt);
size--;
#ifdef DEBUG
printf("\nSize is %i",size);
#endif
infobuffer = (char*) malloc(size);
fread(infobuffer,size,sizeof(char),txt);
fclose(txt);
size = 0;
#ifdef DEBUG
printf("\nTXT File Copied");
#endif
infoplace = infobuffer;

infoplace = SkipLine(infoplace);
strncpy(outstr,infoplace,sizeof(outstr));
FixString(outstr);
#ifdef DEBUG
printf("\nOn PC: %s\n",outstr);
#endif
#ifdef DEBUG
printf("\nGot output filename");
#endif
infoplace = SkipLine(infoplace);
infoplace = SkipLine(infoplace);
infoplace = SkipLine(infoplace);
infoplace = SkipLine(infoplace);
#ifdef DEBUG
printf("\nSkipped pre-level lines\n");
#endif

in = fopen(instr,"rb");
if(!in) return 1;
while(!feof(in))
{
 fgetc(in);
 size++;
}
rewind(in);
size--;
#ifdef DEBUG
printf("\nSize is %i",size);
#endif
levbuffer = (char*) malloc(size);
fread(levbuffer,size,sizeof(char),in);
fclose(in);


levplace = levbuffer;
GetInt(total_maps);
total_maps--; //FIRST (TILEMAP) LEVEL IGNORED
printf("\nNumber of Maps: %i\n",total_maps);
printf("\n----------------------Level Information:-------------------\n");
printf("#  | # Textures | Unique Buttons | Buttons, Doors | Sprites\n");
printf("-----------------------------------------------------------");
{
char *dataptrs[total_maps];
char ceilmodes[total_maps][50];      
char floormodes[total_maps][50];
char *texarray[total_maps];
DoorInfo door[total_maps][MAX_DOORS];
ButtonInfo button[total_maps][MAX_DOORS];
DInfo da[total_maps][MAX_DOORS];
BInfo ba[total_maps][MAX_DOORS];
header = (MAP_HEADER*)malloc(total_maps*sizeof(MAP_HEADER));
#ifdef DEBUG
printf("\nHeader Space Allocated successfully.");
#endif
//================FIRST (TILEMAP) LEVEL IGNORED!===================
LevKeepGoing();
GetInt(curx);
LevKeepGoing();
GetInt(cury);
for(b=0;b<curx*cury*2;b++)
{
  LevKeepGoing();
}
//=================================================================
#ifdef DEBUG
printf("\nFirst Tile Map ignored successfully.");
#endif
for(a=0;a<total_maps;a++)
{
memset(used_textures,0,NR_TEXTURES);
memset(used_buttons,0,NR_TEXTURES);
used_tex = 0;
used_button = 0;
#ifdef DEBUG
printf("\n\n\nCurrent Map Index: %i",a);
#endif
InfoKeepGoing();
strncpy(ceilmodes[a],infoplace,50);
FixString(ceilmodes[a]);
#ifdef DEBUG
printf("\nGot Ceiling Data: %s",ceilmodes[a]);
#endif
InfoKeepGoing();
strncpy(floormodes[a],infoplace,50);
FixString(floormodes[a]);
#ifdef DEBUG
printf("\nGot Floor Data: %s",floormodes[a]);
#endif


infoplace = SkipLine(infoplace);

LevKeepGoing();
GetInt(curx);
LevKeepGoing();
GetInt(cury);
curdoor = 0;
curbutton = 0;
sprite_count = 0;

#ifdef DEBUG
printf("\n[%i,%i] [%X %X]",curx,cury,FlipInt(curx),FlipInt(cury));
#endif

header[a].xsize = FlipInt(curx);
header[a].ysize = FlipInt(cury);
header[a].ceilmode = FlipInt(ParseCeiling(ceilmodes[a]));
header[a].floormode = FlipInt(ParseFloor(floormodes[a]));


#ifdef DEBUG
printf("\nHeader successfully made.");
#endif

dataptrs[a] = (char*) malloc(curx*cury);
#ifdef DEBUG
printf("\nDataptrs[a] successfully allocated.");
#endif
for(b=0;b<curx*cury;b++)
{
// Tile
  LevKeepGoing();
  GetChar(tile);
// Map Code (Enemies, Scripted Events, etc)
  LevKeepGoing();
  GetUChar(mapcode);
  if(tile)
  {
   if(mapcode > 0x40) //Door
   {
     door[a][curdoor].buttoncode = mapcode - 0x40;
     door[a][curdoor].x = b % curx;
     door[a][curdoor].y = b / curx;
     curdoor++;
   }
   else if(mapcode > 0) //Button
   {
     button[a][curbutton].buttoncode = mapcode;
     button[a][curbutton].x = b % curx;
     button[a][curbutton].y = b / curx;
     used_buttons[tile-1] = 1;
     curbutton++;
   }
   dataptrs[a][b] = tile;
   used_textures[tile-1] = 1;
   {
    short y = 0,z;
    for(y=0;y<(signed)(sizeof(animatedwalls)/sizeof(animatedwalls[0]));y++)
    {
     if(animatedwalls[y][0]==tile)
     {
      for(z=1;z<animatedwalls[y][1];z++)
      {
       used_textures[tile-1+z] = 1;
      }
     }
    }
   }
   
   {
   typedef struct
   {
   char map_data[1];
   } Fat;
   typedef struct
   {
   short panel_count;
   short monitor_count;
   } GameGlobals;
   GameGlobals gg;
   GameGlobals* ggg = &gg;
   Fat dfc;
   Fat *fc=&dfc;
   short yy = a;
   short val = tile;
   a = 0;
   fc->map_data[a] = val;
   #include "walls_hit.h"
   if(fc->map_data[a]!=val)
      {
      used_textures[fc->map_data[a]-1] = 1;
      }
      a=yy;
   }
   
   
   }
  else
  {
    for(tmpcount=0;tmpcount<sizeof(levcodearray);tmpcount++)
        if(levcodearray[tmpcount]==mapcode) break;
    if(tmpcount==sizeof(levcodearray) && mapcode) mapcode = 0;
    dataptrs[a][b] = -mapcode;

    if(mapcode >= 0x01 && mapcode < 0x40)
    {
      sprite_count++;
#ifdef DEBUG
printf("\nSprite: %i",mapcode);
#endif
    }
  }
}

{
short t;
for(t=0;t<NR_TEXTURES;t++)
{
  if(used_textures[t]) used_tex++;
  if(used_buttons[t]) used_button++;
/*#ifdef DEBUG
printf("\tT[%i]=%i",t,used_textures[t]);
#endif*/
}
}

#ifdef DEBUG
printf("\n[Header] %X %X %X %X %X %X\n",header[a].xsize,header[a].ysize,
header[a].ceilmode,header[a].floormode,header[a].doorcount,header[a].buttoncount);
#endif


printf("\n%i\t%i/%i\t\t%i/%i\t   %i/%i, %i/%i\t     %i/%i",a+1,used_tex,MAX_TEXTURES,used_button,MAX_UNIQUE_BUTTONS,curbutton,MAX_BUTTONS,curdoor,MAX_DOORS,sprite_count,MAX_SPRITES);

if(curbutton > MAX_BUTTONS)
{
  printf("\nLevel creation failed!");
 printf("\nLevel %i has too many buttons.!\n",a+1);
system("pause");
 QuitOnError();
}
if(curdoor > MAX_DOORS)
{
  printf("\nLevel creation failed!");
 printf("\nLevel %i has too many doors.!\n",a+1);
system("pause");
 QuitOnError();
}
if(sprite_count>MAX_SPRITES)
{
  printf("\nLevel creation failed!");
 printf("\nLevel %i has too many sprites!\n",a+1);
system("pause");
 QuitOnError();
}
if(used_button>MAX_UNIQUE_BUTTONS)
{
  printf("\nLevel creation failed!");
 printf("\nLevel %i has too many unique buttons!",a+1);

 {
  short y = 0;
  printf("\nStarting from index #1:\n");
  for(y = 0;y<NR_TEXTURES;y++)
  {
  if(used_buttons[y]) printf("%i ",y+1);
  }
  printf("\n\n");
 }
system("pause");
 QuitOnError();
}
if(used_tex>MAX_TEXTURES)
{
  printf("\nLevel creation failed!");
 printf("\nLevel %i has too many unique textures!",a+1);
 {
  short y = 0;
  printf("\nStarting from index #1:\n");
  for(y = 0;y<NR_TEXTURES;y++)
  {
  if(used_textures[y]) printf("%i ",y+1);
  }
  printf("\n\n");
 }
system("pause");
 QuitOnError();
}

(header+a)->buttoncount = FlipInt(curbutton);
(header+a)->doorcount = FlipInt(curdoor);
{
 short i,f;
 short FindDoor(short code)
 {
  short t;
  for(t=0;t<curdoor;t++)
     if(door[a][t].buttoncode == code) return t;
  return -1;
 }

 for(i=0;i<curbutton;i++)
 {
  ba[a][i].x = button[a][i].x;
  ba[a][i].y = button[a][i].y;
  if(button[a][i].buttoncode==0x3F)//Keycard
      ba[a][i].x |= 0x80;
  f = FindDoor(button[a][i].buttoncode);
  if(f==-1)
  {
   ba[a][i].doorx = 0;
   ba[a][i].doory = 0;
  }
  else
  {
   ba[a][i].doorx = door[a][f].x;
   ba[a][i].doory = door[a][f].y;
  }
 }
 for(i=0;i<curdoor;i++)
 {
  da[a][i].x = door[a][i].x;
  da[a][i].y = door[a][i].y;
 }
}

fclose(txt);
///////////////-------UNFINISHED SUPPORT FOR COMPRESSED TEXTURES-----------////////////////
#ifdef COMPRESSED_TEXTURES
{
short j;
texarray[a] = malloc(64);
for(j=0;j<header[a].xsize*header[a].ysize;j++)
{

}

}
#endif
////////////////////////////////////////////////////////////////////////////

}
printf("\n");
#ifdef DEBUG
printf("\nCycled through levels successfully.\n");
#endif
#ifdef DEBUG
printf("\nTotal Maps: %i\n",total_maps);
#endif

txt = fopen("archive.txt","wt");
for(a=0;a<total_maps;a++)
{
 sprintf(textstr,"%s_%i.bin",outstr,a);
out = fopen(textstr,"wb");
if(!out)
{
printf("\nFailed opening outfile.\nTried to Open: %s_%i.bin\n",outstr,a);
system("pause");
 return 1;
}
 fwrite(&header[a],sizeof(MAP_HEADER),1,out);
  for(b=0;b<FlipInt(header[a].doorcount);b++)
   fwrite(&da[a][b],sizeof(DInfo),1,out);
  for(b=0;b<FlipInt(header[a].buttoncount);b++)
   fwrite(&ba[a][b],sizeof(BInfo),1,out);
  for(b=0;b<FlipInt(header[a].xsize)*FlipInt(header[a].ysize);b++)
   fwrite(&dataptrs[a][b],sizeof(char),1,out);
  fclose(out);
  sprintf(textstr,"\"..\\tools\\ttpack\" %s_%i.bin %s_%i_.bin >> temp.txt",outstr,a,outstr,a);
  system(textstr);

  fprintf(txt,"%s_%i_.bin %s_%i 0 0\n",outstr,a,outstr,a);
}
  fclose(txt);

if(header) free(header);
for(a=0;a<total_maps;a++)
  if(dataptrs[a]) free(dataptrs[a]);
if(infobuffer) free(infobuffer);
if(levbuffer) free(levbuffer);
#ifdef COMPRESSED_TEXTURES
for(a=0;a<total_maps;a++)
  if(texarray[a]) free(texarray[a]);
#endif

  }

sprintf(textstr,"del temp.txt");
system(textstr);
sprintf(textstr,"\"..\\tools\\ttarchive\" -89 archive.txt -e oblv %s oblivion",outstr);
system(textstr);
sprintf(textstr,"\"..\\tools\\ttarchive\" -92 archive.txt -e oblv %s oblivion",outstr);
system(textstr);
/*sprintf(textstr,"\"..\\tools\\ttbin2oth.exe\" -89 olev %s %s oblivion",outstr,oncalcstr);
system(textstr);
sprintf(textstr,"\"..\\tools\\ttbin2oth\" -92 olev %s %s oblivion",outstr,oncalcstr);
system(textstr);    */
sprintf(textstr,"del *.bin");
system(textstr);
sprintf(textstr,"del archive.txt");
system(textstr);
sprintf(textstr,"move %s.89y ..\\89\\%s.89y",outstr,outstr);
system(textstr);
sprintf(textstr,"move %s.9xy ..\\92p_v200\\%s.9xy",outstr,outstr);
system(textstr);

printf("\nSuccessfully made %s.89y and %s.9xy\n",outstr,outstr);
system("pause");
return 0;
}
